import logging

from zygoat.components import SettingsComponent

log = logging.getLogger()


class AllowedHosts(SettingsComponent):
    def create(self):
        red = self.parse()
        host_list = red.find("name", value="ALLOWED_HOSTS").parent.value

        log.info("Adding allowed host environment config")
        host_list.append("env('DJANGO_ALLOWED_HOST', default='*')")

        log.info("Dumping installed apps node")
        self.dump(red)

    @property
    def installed(self):
        red = self.parse()
        return (
            "DJANGO_ALLOWED_HOST"
            in red.find("name", value="ALLOWED_HOSTS").parent.value.dumps()
        )


allowed_hosts = AllowedHosts()



import logging
import os
import shutil

from .. import Component
from zygoat.utils.files import use_dir
from zygoat.utils.shell import run
from zygoat.constants import Projects, VENV

from .dockerfile import dockerfile
from .wait_command import wait_command
from .settings import settings
from .gitignore import gitignore
from .black import black
from .flake8 import flake8
from .banditrc import banditrc
from .dependencies import dependencies
from .reformat import reformat

log = logging.getLogger()


class Backend(Component):
    def create(self):
        log.info("Installing django at a user level to generate the project")
        run(["pip", "install", "--user", "--upgrade", "django"])

        log.info("Creating the django project")
        run(["django-admin", "startproject", Projects.BACKEND])

        with use_dir(Projects.BACKEND):
            log.info("Creating a virtualenv for the project")
            run(["virtualenv", VENV])

    def update(self):
        pass

    def delete(self):
        log.warning(f"Deleting the {Projects.BACKEND} project")
        shutil.rmtree(Projects.BACKEND)

    @property
    def installed(self):
        return os.path.exists(Projects.BACKEND)


backend = Backend(
    sub_components=[
        settings,
        dockerfile,
        dependencies,
        wait_command,
        gitignore,
        black,
        banditrc,
        flake8,
        # It is important that this comes last, so all relevant items can be reformatted
        reformat,
    ]
)



import logging
import os

from zygoat.components import Component
from zygoat.constants import Phases, Projects, VENV
from zygoat.utils.files import use_dir
from zygoat.utils.shell import run

log = logging.getLogger()


class Reformat(Component):
    def create(self):
        with use_dir(Projects.BACKEND):
            black = os.path.join(VENV, "bin", "black")
            if os.path.exists(black):
                run([black, ".", "--exclude", VENV])

    def update(self):
        self.call_phase(Phases.CREATE, force_create=True)


reformat = Reformat()



import logging

from zygoat.components import SettingsComponent

log = logging.getLogger()


class DatabaseConfig(SettingsComponent):
    def create(self):
        red = self.parse()
        db_node = red.find("name", value="DATABASES").parent

        log.info("Inserting DB URL configuration")
        red.insert(
            red.index(db_node),
            "db_config = env.db_url('DATABASE_URL', default='postgres://postgres:postgres@db/postgres')",
        )

        log.info("Settings database config")
        db_node.value = "{'default': db_config}"

        log.info("Dumping database connection node")
        self.dump(red)

    @property
    def installed(self):
        red = self.parse()
        return "db_config" in red.find("name", value="DATABASES").parent.value.dumps()


database_config = DatabaseConfig()



from functools import partial
import logging

from click import style

from zygoat.utils.files import repository_root
from zygoat.config import Config
from zygoat.constants import Phases

log = logging.getLogger()


class Component:
    """
    The basic unit of project generation in Zygoat.

    Defines lifecycle hooks and handling for configuration + sub-components.
    """

    def __init__(self, parent=None, sub_components=[]):
        """
        Initializes a new component, loads the configuration, and sets the identifier for sub-components.

        :param sub_components:
            Smaller components that make up individual units of this, for organizational distinction.
        :type sub_components: list, optional
        :param parent:
            The "Django ORM"-esque string that identifies the parent component. Usually ``parent.identifier``.

            Set automatically on ``reload`` - **do not manually specify this**.
        :type parent: str, optional
        """
        self.sub_components = []

        self.reload()
        self.parent = parent

        for component in sub_components:
            component.parent = self.identifier
            self.sub_components.append(component)

    @property
    def name(self):
        """
        Convenience function for retrieving the class name
        """
        return self.__class__.__name__

    @property
    def identifier(self):
        """
        The component's path in the tree, using django notation

        (i.e. ``Backend__Linting``, ``Backend__Dockerfile``, ``Frontend__Tests``, etc.)
        """
        if self.parent is None:
            return self.name

        return f"{self.parent}__{self.name}"

    @property
    def exclude(self):
        """
        Whether or not this sub-component should be used in the project
        """
        if self.config.get("exclude", None) is None:
            return False

        return self.identifier in self.config.exclude

    @property
    def include(self):
        """
        Convenience wrapper for easier reading
        """
        return not self.exclude

    def call_phase(self, phase, force_create=False):
        """
        Calls a phase function on self + all sub-components.

        If a component does not have the requested phase, it still executes all sub-components.

        :param phase: The phase to run, from ``zygoat.constants.Phases``
        :type phase: str

        :param force_create: Re-run the ``create`` phase even if the component is installed
        :type force_create: bool, optional
        """
        if self.exclude:
            log.debug(f"Skipping {self.identifier} as it was found in the exclude list")
            return

        self.reload()

        run_self = partial(self._run_self, phase, force_create=force_create)
        run_children = partial(self._run_children, phase, force_create=force_create)

        if phase == Phases.DELETE:
            run_children()
            run_self()
        else:
            run_self()
            run_children()

    @property
    def _log_string(self):
        return "Calling phase {} for {}"

    def _run_self(self, phase, force_create=False):
        phase_func = getattr(self, phase, None)
        is_create = phase == Phases.CREATE
        styled_name = style(self.identifier, bold=True, fg="cyan")

        if phase_func is not None:
            with repository_root():
                if not is_create and not self.installed:
                    log.warning(f"Component {styled_name} is not installed, skipping")
                    return

                if is_create and self.installed and not force_create:
                    log.warning(f"Component {styled_name} is already installed, skipping")
                    return

                log.debug(self._log_string.format(phase, self.__class__.__name__))
                phase_func()

    def _run_children(self, phase, force_create=False):
        for component in self.sub_components:
            log.debug(self._log_string.format(phase, component.__class__.__name__))
            component.call_phase(phase, force_create=force_create)

    @property
    def installed(self):
        """
        Halts the create phase if True
        """
        return False

    def list(self):
        """
        Prints the identifier of this component for exclusion filtering
        """
        if self.include:
            print(self.identifier)

    def reload(self):
        """
        Reloads the configuration file
        """
        self.config = Config()

        for component in self.sub_components:
            component.parent = self.identifier



from zygoat.components import FileComponent


class CodeBuild(FileComponent):
    base_path = "codebuild"


class CodeBuildDefault(CodeBuild):
    filename = "codebuild.yml"


codebuild = CodeBuildDefault()



from zygoat.components import FileComponent
from zygoat.constants import Projects

from . import resources


class NextConfig(FileComponent):
    resource_pkg = resources
    base_path = Projects.FRONTEND
    filename = "next.config.js"
    overwrite = False


next_config = NextConfig()
